#this function read the isoform expression files generated by rSeq
#Return the data matrix object for the use of the main function
#file is a text file that contains the abosulte paths or the file names 
#of the isoform expression files generated by rseq, one file per line
read.rseq=function(file)
{
  files=readLines(file)
  n_sample=length(files)
  cat("Number of samples in total:")
  cat(c(n_sample,'\n'))
  
  file1=read.delim(files[1],header=F,stringsAsFactors=F)
  gene.name=file1[order(file1[,1]),1] #contains gene names, short format
  num.gene=length(gene.name) #this is the number of genes being tested
  cat("Number of genes in total:")
  cat(c(num.gene,'\n'))
  
  isoform.number=file1[,3]#the isoform number for each gene
  iso_name=file1[,4]#isoform name in short format
  
  #below are isoform level--long format
  gene_name=NULL #contains genes names--long format
  isoform_name=NULL #contains isoform names--long format
  isoform_number=NULL #contains the number of isoforms for each gene--long format
  
  for(i in 1:num.gene)
  {
      J=isoform.number[i] #this is the number of isoforms of the gene
      
      gene_name=c( gene_name, rep(gene.name[i],J) )
      
      isoform_name=c(isoform_name, rapply(strsplit(iso_name[i],','), c))
        
      isoform_number=c(isoform_number,rep(isoform.number[i],times=isoform.number[i]))
    
  }
  
  cat("Number of transcripts(isoforms) in total:")
  cat(c(length(isoform_name),'\n'))
  
  isoform.exp=matrix(0,nrow=sum(isoform.number),ncol=n_sample) #this is the expression matrix for all isoforms of all samples
  gene.exp=matrix(0,nrow=num.gene,ncol=n_sample) #this is the expression matrix for all genes of all samples
  
  for(i in 1:n_sample)
  {
    cat(paste("Processing file:",files[i],".\n"))
    temp=read.delim(files[i],header=F,stringsAsFactors=F)
    temp[order(temp[,1]),]
    
    iso_exp=temp[,5] #this is the isoform expression for sample i--short format
        
    iso_exp_temp=NULL #this is the long format isoform expression for sample i
    for(j in 1:num.gene)
    {     
      iso_exp_temp=c(iso_exp_temp, rapply(strsplit(iso_exp[j],','), c))
    }
    
    isoform.exp[,i]=iso_exp_temp    
    
    gene.exp[,i]=temp[,2]
  }  
  
  iso.exp=data.frame(gene_name,isoform_name,isoform_number,isoform.exp) #this is the isoform expression matrix
  gene.exp=data.frame(gene.name,isoform.number=isoform.number,gene.exp)
  
  iso.exp[,1:2] = apply(iso.exp[,1:2], 2, as.character)
  iso.exp[,3:ncol(iso.exp)] = apply(iso.exp[,3:ncol(iso.exp)], 2, as.numeric)
  
  gene.exp[,1] = as.character(gene.exp[,1])
  gene.exp[,2:ncol(gene.exp)] = apply(gene.exp[,2:ncol(gene.exp)], 2, as.numeric)
   
  iso.exp=iso.exp[order(iso.exp[,3],iso.exp[,1],iso.exp[,2]),] #sort by number of isoforms, gene names and isoform names
  gene.exp=gene.exp[order(gene.exp[,2],gene.exp[,1]),] #sort by number of isoforms and isoform names
  
  return(list(iso.exp=iso.exp,gene.exp=gene.exp))
}

read.rsem=function(file, value='FPKM')
{
  #read FPKM or TPM
  if(value=='FPKM')
  {
    col.read=7
  }else if(value=='TPM'){
    col.read=6
  }else{
    stop("Please correctly specify the unit of expression values to be used: FPKM or TPM.")
  }
  
  files=readLines(file)
  n_sample=length(files)
  cat(paste("Number of samples in total: ", n_sample, "\n"))
  
  #use the 1st sample to extract the gene names; isoform names; isoform numbers
  file1_g=paste(files[1],'.genes.results',sep='')
  file1_i=paste(files[1],'.isoforms.results',sep='')
  
  gene.name=read.delim(file1_g,header=T,stringsAsFactors=F)[,1] #contains gene names, short format
  num.gene=length(gene.name) #this is the number of genes being tested
  cat(paste("Number of genes in total: ", num.gene, "\n"))
  
  isoform_info=read.delim(file1_i,header=T,stringsAsFactors=F)[,c(2,1)] #get the gene names and isoform names
  colnames(isoform_info)=c('gene_name','isoform_name')
  
  gene_name=isoform_info[,1]
  num.iso=length(gene_name)
  cat(paste("Number of isoforms in total:", num.iso, "\n"))
  
  gene.info=as.data.frame(table(gene_name)) #get the isoforms numbers for each gene
  colnames(gene.info)=c('gene_name','isoform_number')
  gene.info=gene.info[order(gene.info$gene_name),]
  
  isoform_info=merge(isoform_info,gene.info,by='gene_name')
  isoform_info=isoform_info[order(isoform_info$gene_name, isoform_info$isoform_name),]
    
  iso.exp=matrix(0,nrow=num.iso, ncol=n_sample) #this is the expression matrix for all isoforms of all samples
  gene.exp=matrix(0,nrow=num.gene, ncol=n_sample) #this is the expression matrix for all genes of all samples
  
  for(i in 1:n_sample)
  {
    cat(paste("Processing sample:",files[i],".\n"))
    file_g=paste(files[i],'.genes.results',sep='')
    file_i=paste(files[i],'.isoforms.results',sep='')
    
    g_temp=read.delim(file_g,header=T,stringsAsFactors=F)
    gene.exp[,i]=g_temp[order(g_temp[,1]),col.read]
    
    i_temp=read.delim(file_i,header=T,stringsAsFactors=F)
    iso.exp[,i]=i_temp[order(i_temp[,2], i_temp[,1]),col.read]
  }
  
  iso.exp=data.frame(isoform_info,iso.exp)
  gene.exp=data.frame(gene.info,gene.exp)
  colnames(gene.exp)[1:2]=c('gene.name','isoform.number')
  
  iso.exp[,1:2] = apply(iso.exp[,1:2], 2, as.character)
  iso.exp[,3:ncol(iso.exp)] = apply(iso.exp[,3:ncol(iso.exp)], 2, as.numeric)
  gene.exp[,1] = as.character(gene.exp[,1])
  gene.exp[,2:ncol(gene.exp)] = apply(gene.exp[,2:ncol(gene.exp)], 2, as.numeric)
    
  iso.exp=iso.exp[order(iso.exp[,3],iso.exp[,1],iso.exp[,2]),] #sort by number of isoforms, gene names and isoform names
  gene.exp=gene.exp[order(gene.exp[,2],gene.exp[,1]),] #sort by number of isoforms and isoform names
  
  return(list(iso.exp=iso.exp,gene.exp=gene.exp))  
}
