\name{read.rsem}
\alias{read.rsem}
\title{Read the output files from RSEM}

\description{
This function reads the output files from RSEM (version 1.2.19). The returned list from this function can be used as the input data for \code{rSeqNP.Main} function.
}

\usage{
read.rsem(file, value = "FPKM")
}

\arguments{
  \item{file}{
This is the absolute path of a text file that contains the names of the RNA-Seq samples quantified by RSEM, with one sample name per line. This file should be put in the same folder with the output files from RSEM. For example, if you have a sample named "tumor_sample01" quantified by RSEM, it will generate two result files \code{tumor_sample01.genes.results} and \code{tumor_sample01.isoforms.results}, which are the quantification results for genes and isoforms, respectively. Then you should put "tumor_sample01" in this text file. See the example below and also the documentation of RSEM.}
  \item{value}{
This argument tells which unit of the expression values from the result files of RSEM to be used, can be either \code{TPM} (transcripts per million, which is the sixth column in the result files of RSEM) or  \code{FPKM} (fragments per kilobase of exon per million mapped reads, which is the seventh column in the result files of RSEM). Default value is  \code{FPKM}.}
}

\value{
a list that contains the following two data-frames: 
\item{iso.exp }{a data-frame contains the expression values for the isoforms, with the 1st column as the gene names, the 2nd column as the isoform names, the 3rd column as the number of isoforms corresponding to the genes and the rest of the columns as the expression values (in RPKM or FPKM or other normalized unit) of the isoforms in each sample.
}
\item{gene.exp }{a data-frame contains the expression values for the genes, with the 1st column as the gene names, the 2nd column as the number of isoforms of the genes and the rest columns being the expression values(in RPKM or FPKM or other normalized unit) of the genes in each sample.}
}

\author{Yang Shi and Hui Jiang}

\seealso{
\code{\link{rSeqNP.Main}}
}

\examples{
##read the RSEM processed output files from a simulated RNA-Seq dataset
simulated_data_dir = system.file('extdata','rsem_data_simulated', package='rSeqNP')
setwd(simulated_data_dir)
mydata = read.rsem('samplelist_rsem.txt')
}

\keyword{ process_file }
